@@ -18,19 +18,18 @@ APPVERSION = os.getenv('APPVERSION', "?")
 
 local = os.getenv('KUBERNETES_SERVICE_HOST',"") == ""
 
+
 class NumerousApp():
-    def __init__(self, numerous_client: NumerousClient = None,
+    def __init__(self,
                  appname = "defaultnumerousApp",
                  max_restarts = 0, numerous_job: NumerousBaseJob = None, model_folder: str = None,
-                 working_folder: str = None):
+                 working_folder: str = None, reset_job=False):
         log_level = os.getenv('LOG_LEVEL', logging.root.level)
 
         self.appname = appname
-        self.nc = numerous_client
 
         self.backup = Event()
         self.terminate = Event()
-        self.output = None
         self.scenario = None
         self.model_definition = None
         self.files = None
@@ -51,35 +50,42 @@ class NumerousApp():
         self.numerous_job = numerous_job
         self.result = None
         self.working_folder = working_folder
+        if local:
+            self.client = get_client(reset_job=reset_job)
+        else:
+            self.client = NumerousClient()
+
+        self.client.logger.setLevel(log_level)
+        self.output = DefaultOutputHandler(self.client)
 
         self.setup_scenario()
 
     def setup_scenario(self):
-        self.nc.set_scenario_progress('initializing', ScenarioStatus.ENVIRONMENT_INITIALIZING, 0.0, force=True)
+        self.client.set_scenario_progress('initializing', ScenarioStatus.ENVIRONMENT_INITIALIZING, 0.0, force=True)
 
-        self.scenario, self.model_definition, self.files = self.nc.get_scenario()
+        self.scenario, self.model_definition, self.files = self.client.get_scenario()
 
         main_job = None
         for job in self.scenario.get('jobs', {}).values():
             if job.get('isMain', True):
                 main_job=job
                 break
-        org_job_id = self.nc._job_id
+        org_job_id = self.client._job_id
         if main_job:
             main_job_id = main_job.get('id')
-            self.nc._job_id = main_job_id
+            self.client._job_id = main_job_id
         else:
             self.logger.warning('no main job found on scenario')
 
-        self.nc._run_settings = None
+        self.client._run_settings = None
         # Prepare simulation setup
 
-        self.start_time = self.nc.run_settings.start.timestamp()
-        self.end_time = self.nc.run_settings.end.timestamp()
+        self.start_time = self.client.run_settings.start.timestamp()
+        self.end_time = self.client.run_settings.end.timestamp()
         self.appname = self.scenario['scenarioName']
-        self.allow_hibernation = self.nc.params.get('allow_hibernation', False)
+        self.allow_hibernation = self.client.params.get('allow_hibernation', False)
         self.logger.info(f'allow hibernation: {self.allow_hibernation}')
-        self.nc._job_id = org_job_id
+        self.client._job_id = org_job_id
 
     def loop(self):
 
@@ -95,7 +101,7 @@ class NumerousApp():
             checkpoint_time = datetime.datetime.now()
             if not thread.is_alive() and self.numerous_job._exit_code.value <= ExitCode.ERROR.value:
                 if not warned:
-                    self.nc.set_scenario_progress(message=
+                    self.client.set_scenario_progress(message=
                                                   f"{self.numerous_job._exit_code.description}. Retrying after 5 mins "
                                                   f"({restarts + 1}/{self.max_restarts})",
                                                   status=ScenarioStatus.RUNNING, force=True)
@@ -131,6 +137,13 @@ class NumerousApp():
 
         return self.result
 
+class DefaultOutputHandler:
+    def __init__(self, client: NumerousClient):
+        self.output = client.new_writer(buffer_size=0)
+    def write_row(self, row: dict):
+        self.output.write_row(row)
+
+
 def checkpoint(tnow, tlast, interval=60):
     dt=datetime.timedelta(minutes=interval)
     if tnow-tlast > dt:
@@ -155,21 +168,13 @@ def run_job(numerous_job=None, appname=None, max_restarts=0, model_folder="model
     signal.signal(signal.SIGINT, handler)
     signal.signal(signal.SIGTERM, handler)
 
-    nc = None
     app = None
 
     try:
         if reset_job is None:
             RESET_JOB = os.getenv('RESET_JOB', 'False') == "True"
         else:
-            RESET_JOB = reset_job
-
-        if local:
-            nc = get_client(reset_job=RESET_JOB)
-        else:
-            nc = NumerousClient()
-
-        nc.logger.setLevel(log_level)
+            RESET_JOB = reset_job        
 
         if numerous_job is None:
             raise KeyError("no simulation job specified")
@@ -181,30 +186,30 @@ def run_job(numerous_job=None, appname=None, max_restarts=0, model_folder="model
         #  Create a temporary working folder, that can be used to place generated and downloaded files.
         os.makedirs(working_folder, exist_ok=True)
 
-        app = NumerousApp(numerous_client=nc, appname=appname,
+        app = NumerousApp(appname=appname,
                           max_restarts=max_restarts, numerous_job=numerous_job, model_folder=model_folder,
-                          working_folder=working_folder)
+                          working_folder=working_folder, reset_job=RESET_JOB)
+
 
         app.terminate = terminate
-        nc.terminate_event = terminate
+        app.client.terminate_event = terminate
         app.loop()
 
-        nc.set_scenario_progress(numerous_job._exit_code.description, numerous_job._exit_code.scenariostatus,
+        app.client.set_scenario_progress(numerous_job._exit_code.description, numerous_job._exit_code.scenariostatus,
                                  force=True)
 
         logger.debug(f'setting state {numerous_job._exit_code}')
-        nc.state.set('_status', numerous_job._exit_code.name)
+        app.client.state.set('_status', numerous_job._exit_code.name)
 
     except Exception as e:
         logger.error(f"unhandled exception: {tb_str(e)}")
-        if nc is not None:
-            nc.set_scenario_progress("unhandled error", ScenarioStatus.FAILED, 0.0, force=True)
+        if app and app.client is not None:
+            app.client.set_scenario_progress("unhandled error", ScenarioStatus.FAILED, 0.0, force=True)
             logger.debug(f'setting state -2')
-            nc.state.set('_status', -2)
+            app.client.state.set('_status', -2)
     finally:
-        if nc is not None:
-            nc.close()
-
+        if app and app.client is not None:
+            app.client.close()
         if app:
             #Return the result of the app
             return app.result
