@@ -57,7 +57,7 @@ class NumerousBaseJob:
     def _post_init(self):  # This is called before _run_job from the app.loop method
         timeout = 120
         t, states = self._load_states()
-        dt = self.app.nc.params.get('dt_simulation', 60)
+        dt = self.app.client.params.get('dt_simulation', 60)
 
         #Allow dt parameter to be a time value from front-end
         if isinstance(dt, dict):
@@ -67,14 +67,16 @@ class NumerousBaseJob:
         self._t0 = t
 
 
-        self.input = self.app.nc.get_inputs(self.app.scenario, t0=t, te=t_stop, dt=dt,
+        self.input = self.app.client.get_inputs(self.app.scenario, t0=t, te=t_stop, dt=dt,
                                             tag_prefix='',
                                             timeout=timeout)
 
 
-        self.system = NumerousSystem(self.app.nc, self.app.scenario, self.app.files, self.app.start_time, t_stop,
+        self.system = NumerousSystem(self.app.client, self.app.scenario, self.app.files, self.app.start_time, t_stop,
                                      self.app.model_folder, self, states, t, dt, self.input, wrapper=self.wrapper)
 
+        self.output = self.app.output  # link to app output class
+
     def serialize_states(self, t: float = None) -> Optional[Any]:
         """
         A method called when saving states.
@@ -90,7 +92,7 @@ class NumerousBaseJob:
         model_states = self.serialize_states(t)
         states = {'t': t, 'states': model_states, 'cause': cause, 'details': details}
 
-        self.app.nc.state.set('states', states)
+        self.app.client.state.set('states', states)
 
         self.logger.debug(f"states saved: reason {cause} ({details})")
 
@@ -98,7 +100,7 @@ class NumerousBaseJob:
 
         t = self.app.start_time
         t_end = self.app.end_time
-        states = self.app.nc.state.get('states', {})
+        states = self.app.client.state.get('states', {})
 
         t = states.get('t', t)
 
@@ -141,14 +143,8 @@ class NumerousBaseJob:
         #Overwrite this method to create the simplest job
         raise NotImplementedError
 
-    def handle_output(self, row):
-        if not self.output:
-            self.output = self.app.nc.new_writer(buffer_size=0)
-        self.output.write_row(row)
-
     def post_run(self, exit_code: ExitCode):
-        if self.output:
-            self.output.close()
+        pass
 
 
 class NumerousReportJob(NumerousBaseJob):
@@ -162,7 +158,7 @@ class NumerousReportJob(NumerousBaseJob):
     def run_job(self) -> ExitCode:
         exit_code = ExitCode.COMPLETED
 
-        self.report = Report(self.app.nc.upload_file, template=self.template)
+        self.report = Report(self.app.client.upload_file, template=self.template)
         self.logger.info("adding report content")
         self.add_report_content()
         self.logger.info("finalizing report")
@@ -211,9 +207,9 @@ class NumerousSimulationJob(NumerousBaseJob):
         t = None
         exit_code = ExitCode.COMPLETED
         try:
-            self.app.nc.set_scenario_progress('bootup', ScenarioStatus.INITIALZING, 0.0, force=True)
+            self.app.client.set_scenario_progress('bootup', ScenarioStatus.INITIALZING, 0.0, force=True)
             self.app.terminate.wait(timeout=5)
-            self.app.nc.set_scenario_progress('waiting for initial data', ScenarioStatus.INITIALZING, 0.0, force=True)
+            self.app.client.set_scenario_progress('waiting for initial data', ScenarioStatus.INITIALZING, 0.0, force=True)
 
             t_stop = self.system.end_time
             t = self._t0
@@ -224,10 +220,10 @@ class NumerousSimulationJob(NumerousBaseJob):
             dt_log = 1
 
             while True:
-                if self.app.nc.terminate_event.is_set():
+                if self.app.client.terminate_event.is_set():
                     exit_code = ExitCode.TERMINATED
                     break
-                if self.app.nc.hibernate_event.is_set():
+                if self.app.client.hibernate_event.is_set():
                     exit_code = ExitCode.HIBERNATING
                     break
 
@@ -236,7 +232,7 @@ class NumerousSimulationJob(NumerousBaseJob):
                 except (DataSourceHibernating, TimeoutError) as e:
                     if self.app.allow_hibernation:
                         exit_code = ExitCode.SOURCE_HIBERNATING
-                        self.app.nc.hibernate(message="hibernating")
+                        self.app.client.hibernate(message="hibernating")
                     else:
                         exit_code = -1
                     details = tb_str(e)
@@ -254,7 +250,7 @@ class NumerousSimulationJob(NumerousBaseJob):
                     details = tb_str(e)
                     break
                 except DataSourceStreamStarting:
-                    self.app.nc.set_scenario_progress('waiting for input stream to start',
+                    self.app.client.set_scenario_progress('waiting for input stream to start',
                                                       ScenarioStatus.INITIALZING, 0.0)
                     sleep(0.1)
                     continue
@@ -269,7 +265,7 @@ class NumerousSimulationJob(NumerousBaseJob):
                     self.app.backup.clear()
 
                 if data is None:
-                    self.app.nc.set_scenario_progress(f"waiting for data. Last update: "
+                    self.app.client.set_scenario_progress(f"waiting for data. Last update: "
                                                   f"{datetime.fromtimestamp(t)}",
                                                   ScenarioStatus.RUNNING if not self.init else ScenarioStatus.WAITING,
                                                   completed)
@@ -281,7 +277,7 @@ class NumerousSimulationJob(NumerousBaseJob):
                 self.system.update_inputs(data)
 
                 if t_stop > 0:
-                    self.app.nc.set_scenario_progress("running", ScenarioStatus.RUNNING, completed)
+                    self.app.client.set_scenario_progress("running", ScenarioStatus.RUNNING, completed)
 
                 if t==0 and data['_index'] > 0:
                     self.logger.warning(f'setting start time to {data["_index"]}')
@@ -290,20 +286,20 @@ class NumerousSimulationJob(NumerousBaseJob):
                 # if data is a dict, then convert to list
 
                 if self.init:
-                    self.app.nc.set_scenario_progress("building model", ScenarioStatus.MODEL_INITIALIZING, 0, force=True)
+                    self.app.client.set_scenario_progress("building model", ScenarioStatus.MODEL_INITIALIZING, 0, force=True)
                     initial_output = self.initialize_simulation_system()
                     if initial_output:
                         if "_index" not in initial_output:
                             initial_output.update({'_index': 0})
 
-                        self.handle_output(initial_output)
+                        self.output.write_row(initial_output)
 
                 try:
                     tnew, outputs = self.step(t, self.system.dt)
                     if not outputs:
                         continue
                     if self.init:
-                        self.app.nc.set_timeseries_meta_data([{"name": tag} for tag in outputs.keys()],
+                        self.app.client.set_timeseries_meta_data([{"name": tag} for tag in outputs.keys()],
                                                          offset=self.app.start_time)
                         self.init = False
 
@@ -319,7 +315,7 @@ class NumerousSimulationJob(NumerousBaseJob):
                 # Advance time
                 t = tnew
                 # save output
-                self.handle_output(outputs)
+                self.output.write_row(outputs)
 
                 if time() - last_logged > dt_log:
                     last_logged = time()
